/* sdl-freetype, a text rendering library for SDL.
 * Copyright (C) 2006 luojinghua <sunmoon1997@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it either under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation
 * (the "LGPL") or, at your option, under the terms of the Mozilla
 * Public License Version 1.1 (the "MPL"). If you do not alter this
 * notice, a recipient may use your version of this file under either
 * the MPL or the LGPL.
 *
 * You should have received a copy of the LGPL along with this library
 * in the file COPYING-LGPL-2.1; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * You should have received a copy of the MPL along with this library
 * in the file COPYING-MPL-1.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY
 * OF ANY KIND, either express or implied. See the LGPL or the MPL for
 * the specific language governing rights and limitations.
 *
 * THANKS FOR ALL CAIRO AUTHORS, MOST OF IDEAS AND IMPLEMENT DTAILS ARE
 * STOLEN FROM CAIRO.
 *
 */
#include <stdio.h>
#include "hash.h"
#include "cache.h"
#include "sdl-font.h"

static unsigned long
hash_string(const char * c)
{
    /* This is the djb2 hash. */
    unsigned long hash = 5381;
    while (c && *c)
        hash = ((hash << 5) + hash) + *c++;
    return hash;
}

static unsigned long
cache_entry_hash (const sdl_freetype_hash_entry_t * entry)
{
    const sdl_freetype_cache_entry_t * myentry =
        (const sdl_freetype_cache_entry_t *)entry;

    printf ("compute hash code of entry %p [%s: %lu]\n", entry, 
            (char *)entry->key, myentry->size);

    return hash_string(entry->key);
}

static int
cache_entry_equal (const sdl_freetype_hash_entry_t * entry0,
                   const sdl_freetype_hash_entry_t * entry1)
{
    const sdl_freetype_cache_entry_t * myentry0 =
        (const sdl_freetype_cache_entry_t *)entry0;
    const sdl_freetype_cache_entry_t * myentry1 =
        (const sdl_freetype_cache_entry_t *)entry1;

    printf ("comparing entry %p %p[%s: %lu vs %s:%lu]\n", entry0, entry1,
            (char *)entry0->key, myentry0->size,
            (char *)entry1->key, myentry1->size);

    if (!strcmp(entry0->key, entry1->key))
        return 1;
    return 0;
}

static void
cache_entry_destroy (sdl_freetype_hash_entry_t * entry)
{
    const sdl_freetype_cache_entry_t * myentry =
        (const sdl_freetype_cache_entry_t *)entry;

    printf ("destroying entry %p [%s: %lu]\n", entry,
            (char *)entry->key, myentry->size);
}

static void
cache_entry_itering (sdl_freetype_cache_t * cache,
                     sdl_freetype_cache_entry_t * entry, void * closure)
{
    printf ("itering entry %p [%s: %lu]\n", entry,
            (char *)entry->base.key, entry->size);
}

static sdl_freetype_cache_entry_t cache_entries [] = {
    {{ (void *)"key 1" }, 1},
    {{ (void *)"key 2" }, 1},
    {{ (void *)"key 3" }, 1},
    {{ (void *)"key 4" }, 1},
    {{ (void *)"key 5" }, 1},
    {{ (void *)"key 6" }, 1},
    {{ (void *)"key 7" }, 1},
    {{ (void *)"key 8" }, 1},
    {{ (void *)"key 9" }, 1},
    {{ (void *)"key 10" }, 1},
    {{ (void *)"key 11" }, 1},
    {{ (void *)"key 12" }, 1},
    {{ (void *)"key 13" }, 1},
    {{ (void *)"key 14" }, 1},
    {{ (void *)"key 15" }, 1},
    {{ (void *)"key 16" }, 1},
    {{ (void *)"key 17" }, 1},
    {{ (void *)"key 18" }, 1},
    {{ (void *)"key 19" }, 1},
    {{ (void *)"key 20" }, 1},
};

static int
test_cache (void)
{
    static const sdl_freetype_cache_ops_t hash_ops = {
        cache_entry_hash, cache_entry_equal, cache_entry_destroy
    };
    static const int num_entries = sizeof(cache_entries) / sizeof(cache_entries[0]);

    sdl_freetype_cache_t * cache;
    int i;

    cache = sdl_freetype_cache_create(10, &hash_ops);
    if (!cache)
        return -1;

    for (i = 0; i < num_entries; i++)
        sdl_freetype_cache_insert(cache, &cache_entries[i]);

    if (sdl_freetype_cache_lookup(cache, &cache_entries[3]))
        printf("look %s from hash table successfully\n",
               (char *)cache_entries[3].base.key);

    if (sdl_freetype_cache_lookup(cache, &cache_entries[13]))
        printf("lookup %s from hash table successfully\n",
               (char *)cache_entries[3].base.key);
    else
        printf("lookup %s from hash table failed\n",
               (char *)cache_entries[3].base.key);

    sdl_freetype_cache_foreach(cache, cache_entry_itering, NULL);

    sdl_freetype_cache_freeze(cache);

    for (i = 0; i < num_entries; i++)
        sdl_freetype_cache_insert(cache, &cache_entries[i]);

    sdl_freetype_cache_foreach(cache, cache_entry_itering, NULL);

    sdl_freetype_cache_thaw(cache);

    sdl_freetype_cache_destroy(cache);

    return 0;
}

int
main (int argc, char ** argv)
{
    return test_cache ();
}
