/* sdl-freetype, a text rendering library for SDL.
 * Copyright (C) 2006 luojinghua <sunmoon1997@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it either under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation
 * (the "LGPL") or, at your option, under the terms of the Mozilla
 * Public License Version 1.1 (the "MPL"). If you do not alter this
 * notice, a recipient may use your version of this file under either
 * the MPL or the LGPL.
 *
 * You should have received a copy of the LGPL along with this library
 * in the file COPYING-LGPL-2.1; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * You should have received a copy of the MPL along with this library
 * in the file COPYING-MPL-1.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY
 * OF ANY KIND, either express or implied. See the LGPL or the MPL for
 * the specific language governing rights and limitations.
 *
 * THANKS FOR ALL CAIRO AUTHORS, MOST OF IDEAS AND IMPLEMENT DTAILS ARE
 * STOLEN FROM CAIRO.
 *
 */
#include <stdio.h>
#include "hash.h"
#include "cache.h"
#include "sdl-font.h"

#define FONT "/usr/share/fonts/ttf-bitstream-vera/Vera.ttf"
static int
test_font (void)
{
    sdl_freetype_font_t * font, * font1;
    FT_Face face;
    sdl_freetype_text_extents_t metrics;
    FT_UInt index;
    FT_Int32 ch = ' ';
    FT_Int32 text[] = { 's', 'd', 'l', '-', 'f', 'r', 'e', 'e' };
    FT_Int32 text1[] = { ' ', '\n', '\t' };

    font = sdl_freetype_font_create (FONT, 24);
    if (font)
        printf ("font0 create successfully.\n");
    else
        printf ("font0 create failed.\n");

    font1 = sdl_freetype_font_create (FONT, 32);
    if (font1)
        printf ("font1 create successfully.\n");
    else
        printf ("font1 create failed.\n");

    face = sdl_freetype_font_lock_face (font);
    if (face) {
        printf ("font0 locked successfully.\n");
        sdl_freetype_font_unlock_face (font);
    }

    index = sdl_freetype_font_glyph_index (font, ch);
    printf ("font0 index of `%c' is %d.\n", (char)ch, index);

    if (sdl_freetype_font_glyph_metrics (font, index, &metrics) == 0) {
        printf ("font0 get metrics of `%c' successfully.\n", (char)ch);
    }
    if (sdl_freetype_font_glyph_surface (font, index,
                                         NULL, NULL, NULL, NULL) == 0) {
        printf ("font0 get surface of `%c' successfully.\n", (char)ch);
    }

    sdl_freetype_font_text_extents (font, &metrics, text,
                                    sizeof(text) / sizeof(text[0]));
    sdl_freetype_font_text_extents (font, &metrics, text,
                                    sizeof(text1) / sizeof(text1[0]));

    sdl_freetype_font_show_text (font, NULL, 0xff, 0, 0, 0xff, 0, 0, text,
                                 sizeof(text) / sizeof(text[0]));

    sdl_freetype_font_destroy(font);
    sdl_freetype_font_destroy(font1);

    sdl_freetype_fini();

    return 0;
}

int
main (int argc, char ** argv)
{
    return test_font();
}
