/* sdl-freetype, a text rendering library for SDL.
 * Copyright (C) 2006 luojinghua <sunmoon1997@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it either under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation
 * (the "LGPL") or, at your option, under the terms of the Mozilla
 * Public License Version 1.1 (the "MPL"). If you do not alter this
 * notice, a recipient may use your version of this file under either
 * the MPL or the LGPL.
 *
 * You should have received a copy of the LGPL along with this library
 * in the file COPYING-LGPL-2.1; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * You should have received a copy of the MPL along with this library
 * in the file COPYING-MPL-1.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY
 * OF ANY KIND, either express or implied. See the LGPL or the MPL for
 * the specific language governing rights and limitations.
 *
 * THANKS FOR ALL CAIRO AUTHORS, MOST OF IDEAS AND IMPLEMENT DTAILS ARE
 * STOLEN FROM CAIRO.
 *
 */
#include <stdio.h>
#include "hash.h"
#include "cache.h"
#include "sdl-font.h"

static unsigned long
hash_string(const char * c)
{
    /* This is the djb2 hash. */
    unsigned long hash = 5381;
    while (c && *c)
        hash = ((hash << 5) + hash) + *c++;
    return hash;
}

typedef struct {
    sdl_freetype_hash_entry_t base;
    int value;
} test_hash_entry_t;

static test_hash_entry_t entries [] = {
    {{ (void *)"key 1" }, 0 },
    {{  (void *)"key 2" }, 1 },
    {{  (void *)"key 3" }, 2 },
    {{  (void *)"key 4" }, 3 },
    {{  (void *)"key 5" }, 4 },
    {{  (void *)"key 6" }, 5 },
    {{  (void *)"key 7" }, 6 },
    {{  (void *)"key 8" }, 7 },
    {{  (void *)"key 9" }, 8 },
    {{  (void *)"key 10" }, 9 },
    {{  (void *)"key 11" }, 10 },
    {{  (void *)"key 12" }, 11 },
    {{  (void *)"key 13" }, 12 },
    {{  (void *)"key 14" }, 13 },
    {{  (void *)"key 15" }, 14 },
    {{  (void *)"key 16" }, 15 },
    {{  (void *)"key 17" }, 16 },
    {{  (void *)"key 18" }, 17 },
    {{  (void *)"key 19" }, 18 },
    {{  (void *)"key 20" }, 19 },
};

static unsigned long
hash_entry_hash (const sdl_freetype_hash_entry_t * entry)
{
    const test_hash_entry_t * myentry = (const test_hash_entry_t *)entry;

    printf ("compute hash code of entry %p [%s: %d]\n", entry, 
            (char *)entry->key, myentry->value);

    return hash_string(entry->key);
}

static int
hash_entry_equal (const sdl_freetype_hash_entry_t * entry0,
                  const sdl_freetype_hash_entry_t * entry1)
{
    const test_hash_entry_t * myentry0 = (const test_hash_entry_t *)entry0;
    const test_hash_entry_t * myentry1 = (const test_hash_entry_t *)entry1;

    printf ("comparing entry %p %p[%s: %d vs %s:%d]\n", entry0, entry1,
            (char *)entry0->key, myentry0->value,
            (char *)entry1->key, myentry1->value);

    if (!strcmp(entry0->key, entry1->key))
        return 1;
    return 0;
}

static void
hash_entry_destroy (sdl_freetype_hash_entry_t * entry)
{
    const test_hash_entry_t * myentry = (const test_hash_entry_t *)entry;

    printf ("destroying entry %p [%s: %d]\n", entry,
            (char *)entry->key, myentry->value);
}

static void
hash_entry_itering (sdl_freetype_hash_t * hash,
                    sdl_freetype_hash_entry_t * entry, void * closure)
{
    const test_hash_entry_t * myentry = (const test_hash_entry_t *)entry;

    printf ("itering entry %p [%s: %d]\n", entry,
            (char *)entry->key, myentry->value);
}

static int
test_hash (void)
{
    static const sdl_freetype_hash_ops_t hash_ops = {
        hash_entry_hash, hash_entry_equal, hash_entry_destroy
    };
    static const int num_entries = sizeof(entries) / sizeof(entries[0]);

    sdl_freetype_hash_t * hash;
    int i;

    hash = sdl_freetype_hash_create(10, &hash_ops);
    if (!hash)
        return -1;

    for (i = 0; i < num_entries; i++)
        sdl_freetype_hash_insert(hash, &entries[i].base);

    if (sdl_freetype_hash_lookup(hash, &entries[3].base))
        printf("look %s from hash table successfully\n", 
               (char *)entries[3].base.key);

    sdl_freetype_hash_remove(hash, &entries[3].base);

    if (sdl_freetype_hash_lookup(hash, &entries[3].base))
        printf("lookup %s from hash table successfully\n",
               (char *)entries[3].base.key);
    else
        printf("lookup %s from hash table failed\n",
               (char *)entries[3].base.key);

    sdl_freetype_hash_foreach(hash, hash_entry_itering, NULL);
    
    sdl_freetype_hash_resize(hash);

    for (i = 0; i < num_entries; i++)
        sdl_freetype_hash_insert(hash, &entries[i].base);

    sdl_freetype_hash_foreach(hash, hash_entry_itering, NULL);

    sdl_freetype_hash_destroy(hash);

    return 0;
}

int
main (int argc, char ** argv)
{
    return test_hash ();
}
