/* sdl-freetype, a text rendering library for SDL.
 * Copyright (C) 2007 luojinghua <sunmoon1997@gmail.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it either under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation
 * (the "LGPL") or, at your option, under the terms of the Mozilla
 * Public License Version 1.1 (the "MPL"). If you do not alter this
 * notice, a recipient may use your version of this file under either
 * the MPL or the LGPL.
 *
 * You should have received a copy of the LGPL along with this library
 * in the file COPYING-LGPL-2.1; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * You should have received a copy of the MPL along with this library
 * in the file COPYING-MPL-1.1
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY
 * OF ANY KIND, either express or implied. See the LGPL or the MPL for
 * the specific language governing rights and limitations.
 *
 * THANKS FOR ALL CAIRO AUTHORS, MOST OF IDEAS AND IMPLEMENT DTAILS ARE
 * STOLEN FROM CAIRO.
 *
 */
#ifndef SDL_FREETYPE_AREA_MANAGER_H
#define SDL_FREETYPE_AREA_MANAGER_H

#include "sdl-freetype.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct sdl_freetype_area_manager_s sdl_freetype_area_manager_t;
typedef struct sdl_freetype_area_s sdl_freetype_area_t;

#define SDL_FREETYPE_AREA_AVAILABLE 0
#define SDL_FREETYPE_AREA_DIVIDED   1
#define SDL_FREETYPE_AREA_OCCUPIED  2

struct sdl_freetype_area_s {
    int state;
    int level;
    int x, y;
    int width, height;
    sdl_freetype_area_t * area[4];
    void * closure;
};

typedef struct {
    sdl_freetype_error_t
    (* move_in)(sdl_freetype_area_manager_t * manager,
                sdl_freetype_area_t * area, void * closure);

    void
    (* move_out)(sdl_freetype_area_manager_t * manager,
                 sdl_freetype_area_t * area, void * closure);

    int
    (* compare_score) (sdl_freetype_area_manager_t * manager,
                       sdl_freetype_area_t * area,
                       void * closure1, void * closure2);
} sdl_freetype_area_manager_ops_t;

sdl_freetype_area_manager_t *
sdl_freetype_area_manager_create(int width, int height, int max_level,
                                 const sdl_freetype_area_manager_ops_t * ops);

sdl_freetype_error_t
sdl_freetype_area_manager_find (sdl_freetype_area_manager_t * manager,
                                int                           width,
                                int                           height,
                                sdl_freetype_bool_t           kick_out,
                                void                        * closure);

void
sdl_freetype_area_manager_move_out (sdl_freetype_area_manager_t * manager,
                                    sdl_freetype_area_t         * area);

void
sdl_freetype_area_manager_destroy(sdl_freetype_area_manager_t * manager);

#ifdef __cplusplus
}
#endif

#endif
