/* Copyright (C) <2011> Luo Jinghua<sunmoon1997@gmail.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#include <SDL.h>
#include <GL/gl.h>

#include <sdl-freetype.h>
#include <sdl-freetype-opengl.h>
#include <sdl-ft.h>

#include <stdio.h>

#define TEXT "sdl freetype OpenGL render"

int
main(int argc, char *argv[])
{
    SDL_Surface * screen;
    Uint8 video_bpp;
    Uint32 videoflags;
    int done;
    SDL_Event event;
    float alpha, step;
    sdl_freetype_font_t * font;
    sdl_freetype_glyph_render_t * render;
    sdl_freetype_text_extents_t extents;
    short utf16[] = { 0x4f60, 0x597d, 0xff01, 0x0000 };

    /* Initialize SDL */
    if (SDL_Init (SDL_INIT_VIDEO) < 0)
        goto errquit0;

    videoflags = SDL_HWSURFACE;
    videoflags |= SDL_OPENGL;
    video_bpp = 32;

    /* Set 640x480 video mode */
    screen = SDL_SetVideoMode (640, 480, video_bpp, videoflags);
    if (!screen)
        goto errquit0;

    font = sdl_ft_create_font ("sans", 20,
                               SDL_FT_WEIGHT_NORMAL, SDL_FT_SLANT_NORMAL, 96);
    if (!font)
        goto errquit0;

    render = sdl_freetype_opengl_render_create ();
    if (!render)
        goto errquit1;
    sdl_freetype_font_set_render (font, render);

    glMatrixMode (GL_PROJECTION) ;
    glLoadIdentity ();
    glOrtho (0, 640, 480, 0, -1.0, 1.0);

    glMatrixMode (GL_MODELVIEW);
    glLoadIdentity ();

    glEnable (GL_TEXTURE_2D);
    glClearColor (0.8f, 0.8f, 0.8f, 0.0f);

    alpha = 1.0f;
    step = -0.01;
    /* Wait for a keystroke */
    done = 0;
    while (!done) {
        /* Check for events */
        while (SDL_PollEvent (&event)) {
            switch (event.type) {
            case SDL_MOUSEBUTTONDOWN:
                break;
            case SDL_KEYDOWN:
            case SDL_QUIT:
                done = 1;
                break;
            default:
                break;
            }
        }

        glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glMatrixMode (GL_MODELVIEW);
	glLoadIdentity ();

        sdl_freetype_font_utf8_extents (font, &extents, TEXT, -1);
        glTranslatef ((640 - extents.width) / 2 - extents.x_bearing,
                      (480 - extents.height) / 2 + extents.y_bearing,
		      0);
        sdl_freetype_font_show_utf8 (font, NULL, 255, 0, 0, alpha * 255,
                                     0, 0, TEXT, -1);
	glLoadIdentity ();
	glTranslatef (100.0f, 100.0f, 0.0f);
        sdl_freetype_font_show_utf16 (font, NULL, 255, 0, 0, alpha * 255,
				      0, extents.height, utf16, -1);

        SDL_GL_SwapBuffers();

        alpha += step;
        if (alpha < 0.0f) {
            alpha = 0.0f;
            step = -step;
        } else if (alpha > 1.0f) {
            alpha = 1.0f;
            step = -step;
        }

        SDL_Delay(20);
    }

 errquit1:
    sdl_freetype_font_destroy (font);
    sdl_freetype_fini ();

 errquit0:
    SDL_Quit();
    return 0;
}
